/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include <sensors/sensors.h>
#include <sensors/error.h>

#include "config.h"
#include "psensor.h"
#include "graph.h"
#include "ui_sensorlist.h"
#include "nvidia.h"

#ifndef PSENSOR_VERSION
#define PSENSOR_VERSION "0.0.0"
#endif

#ifndef PSENSOR_DIR
#define PSENSOR_DIR "/usr/share/psensor"
#endif

#define PSENSOR_ICON_FILENAME "psensor-32x32.png"

struct ui_psensor {	
	struct psensor** sensors;

	GtkWidget* w_graph;

	struct ui_sensorlist* ui_sensorlist;	

	struct color* graph_bgcolor;
};

void 
update_information(struct psensor** sensors) 
{
	while(1) {
		struct psensor** sensor_cur = sensors;
		while(*sensor_cur) {
			gdk_threads_enter();
			update_psensor(*sensor_cur);
			gdk_threads_leave();
			sensor_cur++;
		}
		sleep(1);
	}
}

gboolean 
refresh_thread(gpointer data) 
{
	struct ui_psensor* ui_psensor = (struct ui_psensor*)data;

	update_graph(ui_psensor->sensors,
		     ui_psensor->w_graph,
		     ui_psensor->graph_bgcolor);

	ui_sensorlist_update(ui_psensor->ui_sensorlist);

	return TRUE;
}

gboolean 
on_expose_event (GtkWidget *widget, 
			  GdkEventExpose *event, 
			  gpointer data) 
{
	struct ui_psensor* ui_psensor = (struct ui_psensor*)data;

	update_graph(ui_psensor->sensors, 
		     ui_psensor->w_graph,
		     ui_psensor->graph_bgcolor);

	return FALSE;
}

void 
on_destroy(GtkWidget *widget,
	     gpointer data )
{
    gtk_main_quit ();
}

GdkPixbuf *
create_pixbuf(const gchar* filename)
{
   GdkPixbuf *pixbuf;
   GError *error = NULL;

   pixbuf = gdk_pixbuf_new_from_file(filename, &error);

   if(pixbuf) {
	   return pixbuf;
   } else {
      fprintf(stderr, "%s\n", error->message);
      g_error_free(error);
      return NULL;
   }
}

GdkPixbuf* 
create_psensor_icon()
{
	int pathlength
		= strlen(PSENSOR_DIR)+1+strlen(PSENSOR_ICON_FILENAME)+1;
	char* path = malloc(pathlength);

	strcpy(path, PSENSOR_DIR);

	if (PSENSOR_DIR[strlen(PSENSOR_DIR)-1] != '/') 
		strcat(path, "/");

	strcat(path, PSENSOR_ICON_FILENAME);

	GdkPixbuf* pixbuf = create_pixbuf(path);

	free(path);

	return pixbuf;
}

void
associate_colors(struct psensor** sensors)
{
	// number of uniq colors
	#define COLORS_COUNT 8
	
	unsigned int colors[COLORS_COUNT][3] = { 
		{0x0000, 0x0000, 0x0000},// black
		{0xffff, 0x0000, 0x0000},// red
		{0x0000, 0.0000, 0xffff},// blue
		{0x0000, 0xffff, 0x0000},// green
		
		{0x7fff, 0x7fff, 0x7fff},// grey
		{0x7fff, 0x0000, 0x0000},// dark red
		{0x0000, 0x0000, 0x7fff},// dark blue
		{0x0000, 0x7fff, 0x0000} // dark green
	};


	struct psensor** sensor_cur = sensors;
	int i = 0;
	while(*sensor_cur) {
		struct color default_color;
		color_set(&default_color,
			  colors[i%COLORS_COUNT][0],
			  colors[i%COLORS_COUNT][1],
			  colors[i%COLORS_COUNT][2]);
		
		(*sensor_cur)->color 
			= config_get_sensor_color((*sensor_cur)->id, 
						  &default_color);
		
		sensor_cur++;
		i++;
	}
}

int 
on_graph_clicked(GtkWidget* widget, 
		 GdkEventButton* event,
		 gpointer data)
{
	struct ui_psensor* uipsensor = (struct ui_psensor*)data;

	GdkColor color;
	color.red = uipsensor->graph_bgcolor->red;
	color.green = uipsensor->graph_bgcolor->green;
	color.blue = uipsensor->graph_bgcolor->blue;
	
	GtkColorSelectionDialog* colordlg
		= (GtkColorSelectionDialog*)
		gtk_color_selection_dialog_new("Select background color");

	GtkColorSelection* colorsel 
		= GTK_COLOR_SELECTION(colordlg->colorsel);
	
	gtk_color_selection_set_previous_color(colorsel, &color);
	gtk_color_selection_set_current_color(colorsel, &color);
	
	int res = gtk_dialog_run (GTK_DIALOG (colordlg));

	if (res == GTK_RESPONSE_OK) {
		gtk_color_selection_get_current_color(colorsel, 
						      &color);
		
		color_set(uipsensor->graph_bgcolor,
			  color.red,
			  color.green,
			  color.blue);

		config_set_background_color(uipsensor->graph_bgcolor);
	}
	
	gtk_widget_destroy(GTK_WIDGET(colordlg));

	return FALSE;
}


int 
main(int argc, char** argv) 
{
	struct ui_psensor ui_psensor;

	printf("Psensor - Temperature Monitor - v%s by wpitchoune@gmail.com\n",
	       PSENSOR_VERSION);
	printf("Released under GPLv2 license terms, read LICENSE file\n");
	printf("http://wpitchoune.net:9969/psensor\n");

	g_thread_init(NULL);
	gdk_threads_init();
	gdk_threads_enter();

	config_init();

	ui_psensor.graph_bgcolor = config_get_background_color();

	int err = sensors_init(NULL);
	if (err) {
		fprintf(stderr, "ERROR: %s\n", sensors_strerror(err));
		return 1;
	}

	ui_psensor.sensors = get_all_sensors();
	associate_colors(ui_psensor.sensors);


	gtk_init (&argc, &argv);

	GtkWidget* window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width (GTK_CONTAINER (window), 2);
	gtk_window_set_default_size(GTK_WINDOW(window),800,200);
	gtk_window_set_title(GTK_WINDOW(window), 
			     "Psensor - Temperature Monitor");

	GdkPixbuf* icon = create_psensor_icon();
	if (icon)
		gtk_window_set_icon(GTK_WINDOW(window), 
				    icon);
	
	g_signal_connect (window, "destroy",
			  G_CALLBACK(on_destroy),
			  NULL);

	//////////// main box in the window
	GtkWidget* mbox = gtk_hpaned_new();
	gtk_container_add (GTK_CONTAINER (window), mbox);

	//////////// drawing box
	GtkDrawingArea* drawing_area 
		= GTK_DRAWING_AREA(gtk_drawing_area_new ());
	ui_psensor.w_graph = GTK_WIDGET(drawing_area);

	gtk_paned_pack1(GTK_PANED(mbox), 
			GTK_WIDGET(drawing_area), 
			TRUE, TRUE);

	g_signal_connect(G_OBJECT(drawing_area), 
			 "expose-event",
			 G_CALLBACK(on_expose_event), &ui_psensor);

	gtk_widget_add_events(GTK_WIDGET(drawing_area), GDK_BUTTON_PRESS_MASK);
	gtk_signal_connect(GTK_OBJECT(drawing_area), 
			   "button_press_event", 
			   (GCallback)on_graph_clicked, 
			   &ui_psensor);



	///////////// sensor list
	ui_psensor.ui_sensorlist = ui_sensorlist_create(ui_psensor.sensors);


	GtkWidget* scrolled_window 
		= gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				       GTK_POLICY_NEVER, 
				       GTK_POLICY_AUTOMATIC);

	gtk_container_add (GTK_CONTAINER(scrolled_window), 
			   ui_psensor.ui_sensorlist->widget);

	gtk_paned_pack2(GTK_PANED(mbox), 
			scrolled_window, 
			FALSE, TRUE);


	gtk_widget_show_all (window);

	GError* error;
	GThread* thread = g_thread_create((GThreadFunc)update_information,
					  ui_psensor.sensors, TRUE, &error);

	if(!thread) 
		g_error_free(error);
	
	g_timeout_add (1000, refresh_thread, &ui_psensor);

	gtk_main ();
	gdk_threads_leave();

	sensors_cleanup();

	psensor_list_free(ui_psensor.sensors);

	free(ui_psensor.graph_bgcolor);

	return 0;
}
